/*  $Id: StrucFindDialog.h,v 1.29 2003/05/22 16:43:09 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file 
    Declarations for the <code>StrucFindDialog</code> class.
    @author Holger Wunsch */

#ifndef StrucFindDialog_included
#define StrucFindDialog_included

#include "StrucFindDialogData.h"

#include <qstring.h>

/** The <code>StrucFindDialog</code> class implements the Find dialog.
    @author Holger Wunsch */
class StrucFindDialog : public StrucFindDialogData
{
  Q_OBJECT


  /** The accelerator that handles the CTRL+G key (alternative shortcut for
      Find Next). */
  QAccel* accel_CtrlG;

  /** Positions the dialog to make sure that it doesn't cover the current
      view. */
  void positionDialog(void);

 public:
  /** Creates a new find dialog. The "Find Next" button is initially disabled,
      "Ignore Case" is checked.
      @param title the dialog title
      @param parent the parent window
      @param name the dialog's name (internal use only) */
  StrucFindDialog(const QString& title, QWidget* parent = NULL, const char* name = NULL);


  /** Creates a new find dialog with a pre-set search term. The "Find Next" button
      is initially enabled, "Ignore Case" is unchecked.
      @param title the dialog title
      @param searchTerm the search term.
      @param parent the parent window
      @param name the dialog's name (internal use only) */
  StrucFindDialog(const QString& title, char* searchTerm, QWidget* parent = NULL, const char* name = NULL);
  

  /** Destroys the find dialog. */
  virtual ~StrucFindDialog();
  

  /** Returns the search term. */
  const QString text();


  /** Returns the status of the "Ignore Case" checkbox.
      @return <code>true</code> if "Ignore Case" is checked, <code>false</code> otherwise. */
  bool ignoreCaseChecked(void);
  
 signals:
  /** This signal is raised when the user clicks the "Find" button. */
  void findSignal();


  /** This signal is raised when the user clicks the "Find Next" button. */
  void findNextSignal();


  /** This signal is raised when the user clicks the "Cancel" button. */
  void cancelSignal();
  
 protected:
  /** Called when the user clicked the "Find" button. */
  virtual void okSlot();


  /** Called when the user clicked the "Cancel" button. */
  virtual void cancelSlot();


  /** Called when the user clicked the "Find Next" button. */
  virtual void nextSlot();
  

  /** Called when the user entered a new search term.
      @param text the new search term */
  virtual void textChangedSlot(const QString& text);
  
  
  /** Called when the user checked or unchecked the
      ignore case checkbox.
      @param on the new value of the check box. */
  virtual void ignoreCaseToggledSlot(bool on);
  

  /** Called when the dialog is closed.
      @param ev the close event. */
  void closeEvent(QCloseEvent* ev);
  
 protected slots:
   /** Called when the user clicked the "Cancel" button. */
  virtual void reject();
    
};
#endif // StrucFindDialog_included
