/*  $Id: dataparser.h,v 1.33 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file
    Declarations for the DataParser class.
    @author Holger Wunsch */

#ifndef __DATAPARSER_H
#define __DATAPARSER_H

#include "node.h"
#include "data.h"


/** The DataParser class implements the parser that is used by Grisu to parse incoming or
    loaded data packages and to build the internal Node based data representation. DataParser is a wrapper around 
    the Bison-generated parser in <code>il.y</code>.
    @author Holger Wunsch */
class DataParser {
 public:

  /** Parses a data package.
      @param ioBuffer a buffer that contains a <b>complete</b> data package.
      @return a Document that contains the data. */
  Document* parse(char* ioBuffer);
};


/** This structure is used to pass several input and output arguments to the Bison parser. */
struct ParserArgs {
  /** A list of top level nodes. The first element in this list must be the root node of the main
      feature structure node (StrucNode), followed by more structure nodes, one TreeNode and
      other types of nodes as specified in the interface EBNF. */
  List* parsedData;

  
  /** The string that should appear in the title bar of a view window that shows the feature structure. */
  char* windowtitle;


  /** A mapping between node IDs and nodes. */
  NodeIdMap* map;


  /** The I/O buffer. */
  char* ioBuffer;


  /** One of the <code>DRAW_xxxx</code> values, specifies how a structure should be shown in
      a view window depending on the argument to <code>!newdata</code> in the data package. */
  int showDefault;
};


/** The main entry point of the parser.
    @param args a pointer to a ParserArgs structure.
    @return see the Bison documentation at 
    <a href="http://www.gnu.org/software/bison/bison.html">http://www.gnu.org/software/bison/bison.html</a>. */
int yyparse(void* args);
  

#endif
