/*  $Id: drawer.cpp,v 1.34 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of all drawing routines of the <code>Canvas</code> class.
    All other routines are implemented in <code>canvas.cpp</code>.

    @author Holger Wunsch */


#include <qdir.h>
#include <qcursor.h>
#include <qpainter.h>
#include <qstring.h>
#include <ctype.h>

#include "canvas.h"
#include "global.h"

#ifdef _LINUX
#include <ctype.h>
#endif


void Canvas::drawText(int x, int y, const char *t, int width) {
  // ignore width !!!!!
  if (realDraw)
    paint->drawText(x, y + paint->fontMetrics().ascent(), QString::fromLocal8Bit(t));
}


void Canvas::drawLine(int x1, int y1, int x2, int y2) {
  if (realDraw)
    paint->drawLine(x1, y1, x2, y2);
}


void Canvas::drawRect(int x1, int y1, int x2, int y2) {
  if (realDraw)
    paint->drawRect(x1, y1, x2, y2);
}


void Canvas::drawAndOperator(int x, int width, int yu, int yd) {
  if (realDraw) {
    paint->drawLine(x, yd, x + width / 2, yu);
    paint->drawLine(x + width / 2, yu, x + width, yd);
  }
}


void Canvas::drawOrOperator(int x, int width, int yu, int yd) {
  if (realDraw) {
    paint->drawLine(x, yu, x + width / 2, yd);
    paint->drawLine(x + width / 2, yd, x + width, yu);
  }
}


void Canvas::addToClick(const NodeAddress& addr, int nodetype, QRect bounds) {
  KlickBox* click = new KlickBox;

  click->addr = addr;
  click->type = nodetype;
  click->box = bounds;

  toKlick.append(click);
}
  

/** Converts a string into upper case (in-place).
    @param str the string to be converted. The string will be modified. */
void string_upper(char* str) {
  char* ptr = str;

  while (*ptr) {
    *ptr = toupper(*ptr);
    ptr++;
  }
}

void Canvas::paintEvent(QPaintEvent *) {
  QCursor oldCursor = cursor();
  setCursor(waitCursor);
  
  resetToClick();

  //Ok in the moment we redraw EVERYTHING!
  // 
  // pm is our doublebuffer for flickerfree displaing
  QPixmap pm( size() );
  pm.fill(options->backgroundColor);   //backgroundColor());

  // The painter to draw
  paint=new QPainter;
  paint->begin(&pm);

  // Not only the size - no we want a REAL picture :-)
  realDraw=true;

  // Set the maximum size for the scrollbars ...
  //  QString rpath(rootPath);
  //  maxSize = drawNode(rootNode, rpath, 5, 5, false);

  maxSize = drawGenericNode(rootNode, NodeAddress(), 5 - offsetX, 5 - offsetY);

  RelationNode* nd = document->rel->first();
  QSize sz;
  int nexty = 5 - offsetY + maxSize.height();
  while (nd) {
    nexty += 2 * options->offset;
    sz = drawGenericNode((GenericNode*) nd, NodeAddress(), 5 - offsetX, nexty);
    nexty += sz.height();
    maxSize.setWidth(max(maxSize.width(), sz.width()));
    nd = document->rel->next();
  }

  maxSize.setHeight(nexty - (5 - offsetY));

  paint->end();
  delete paint;
  // doublebuffer back to display.
  bitBlt(this,0,0,&pm);

  int wi=maxSize.width()+15-width();
  int he=maxSize.height()+15-height();
  if(wi<0) wi=0;
  if(he<0) he=0;
  h_scroll->setRange(0, wi);
  v_scroll->setRange(0, he);
  h_scroll->setSteps(10, width());
  v_scroll->setSteps(10, height());
  h_scroll->setValue(offsetX);
  v_scroll->setValue(offsetY);

  setCursor(oldCursor);
}

void Canvas::printToPrinter(QPrinter &printer) {
  int oldOffsetX = offsetX;
  int oldOffsetY = offsetY;
  
  offsetY = offsetX = 0;

  // The draw painter to draw
  paint=new QPainter;
  paint->begin(&printer);

  realDraw=false;

  maxSize=drawGenericNode(rootNode, NodeAddress(), 5, 5);

  RelationNode* nd = document->rel->first();
  QSize sz;
  int nexty = 5 + maxSize.height();
  while (nd) {
    nexty += 2 * options->offset;
    sz = drawGenericNode((GenericNode*) nd, NodeAddress(), 5, nexty);
    nexty += sz.height();
    maxSize.setWidth(max(maxSize.width(), sz.width()));
    nd = document->rel->next();
  }

  maxSize.setHeight(nexty - (5 - offsetY));
  // Compute the window-size
  QRect win = paint->window();
  double xfac =1;
  double yfac =1;
  if (win.width() < maxSize.width()) 
    xfac = double(maxSize.width()) / double(win.width());
  if (win.height() < maxSize.height()) 
    yfac = double(maxSize.height()) / double(win.height());

  double fac = xfac;

  if(yfac > xfac)
    fac=yfac;


  paint->setWindow(0,0,int(win.width()*fac),int(win.height()*fac));

  realDraw = true;

  maxSize=drawGenericNode(rootNode, NodeAddress(), 5, 5);

  nd = document->rel->first();
  nexty = 5 + maxSize.height();
  while (nd) {
    nexty += 2 * options->offset;
    sz = drawGenericNode((GenericNode*) nd, NodeAddress(), 5, nexty);
    nexty += sz.height();
    maxSize.setWidth(max(maxSize.width(), sz.width()));
    nd = document->rel->next();
  }

  maxSize.setHeight(nexty - (5 - offsetY));
  paint->end();

  delete paint;

  offsetX = oldOffsetX;
  offsetY = oldOffsetY;

  paintEvent(NULL);
}




void Canvas::repaint(bool redraw) {
  // Since we always update the whole screen we never have to 
  // delete it before ....
  paintEvent(NULL);
}


void Canvas::repaint(int, int, int, int, bool redraw) {
  // Clipping would be too much work.
  // So we redraw EVERYTHING :-)
  repaint(redraw);
}

QSize Canvas::getNodeDimensions(GenericNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  QSize ndSize;
  bool realDrawSave = realDraw;

  realDraw = false;

  ndSize = drawGenericNode(nd, addrParent, dx, dy);

  realDraw = realDrawSave;

  return ndSize;
}

QSize Canvas::drawGenericNode(GenericNode* nd, const NodeAddress& addrParent, int dx, int dy, unsigned short flags) {
  QSize ndSize(0, 0);

  switch (nd->nodetype) {
  case NODETYPE_TYPE_NODE:
    ndSize = drawTypeNode((TypeNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_STRUC_NODE:
    ndSize = drawStrucNode((StrucNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_FEATVAL_NODE:
    ndSize = drawFeatvalNode((FeatvalNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_LIST_NODE:
    ndSize = drawListNode((ListNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_SET_NODE:
    ndSize = drawSetNode((SetNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_FUNCTION_NODE:
    ndSize = drawFunctionNode((FunctionNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_REL_NODE:
    ndSize = drawRelationNode((RelationNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_REF_NODE:
    ndSize = drawRefNode((RefNode*) nd, addrParent, dx, dy, flags);
    break;
  case NODETYPE_TREE_NODE:
    ndSize = drawTreeNode((TreeNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_ATOM_NODE:
    ndSize = drawAtomNode((AtomNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_TAIL_NODE:
    ndSize = drawTailNode((TailNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_REST_NODE:
    ndSize = drawRestNode((RestNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_DISJUNCTION_NODE:
    ndSize = drawDisjunctionNode((DisjunctionNode*) nd, addrParent, dx, dy);
    break;
  case NODETYPE_CONJUNCTION_NODE:
    ndSize = drawConjunctionNode((ConjunctionNode*) nd, addrParent, dx, dy);
    break;
  default:
    printf("Unsupported node type %d\n", nd->nodetype);
    break;
  }

  NodeAddress addr(addrParent, (GenericNode*) nd);

  if (realDraw) {
    if (options->highlightStructures && highlight && (highlight->addr == addr)) {
      paint->save();
      paint->setPen(options->markerColor);
      drawRect(dx - 1, dy - 1,
	       ndSize.width() + 2, ndSize.height() + 2);
      paint->restore();
    }
  }

  return ndSize;
}

void Canvas::drawFoundMark(int x, int y, int w, int h) {
  paint->save();
  paint->setPen(options->foundColor);
  drawRect(x - 1, y - 1,
	   w + 2, h + 2);
  paint->restore();
}

QSize Canvas::drawHiddenNode(DisplayableNode* dnd, char* text, int dx, int dy) {
  QSize ndSize;
  char buf[255];

  if (options->drawHidden || (dnd->flags & NODEFLAG_SHOWHIDDEN)) {

    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->hiddenColor);

    QFontMetrics fm = paint->fontMetrics();

    sprintf(buf, "%s", text);

    drawText(dx, dy, buf);
    
    ndSize.setWidth(fm.width(buf));
    ndSize.setHeight(fm.height());

    paint->restore();

    return ndSize;
  }
  else {
    ndSize.setWidth(0);
    ndSize.setHeight(0);
    return ndSize;
  }
}   

QSize Canvas::drawTypeNode(TypeNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, nd->name, dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->typeFont);
    paint->setPen(options->typeColor);
    paint->setBackgroundMode(OpaqueMode);

    QFontMetrics fm = paint->fontMetrics();

    ndSize.setWidth(fm.width(nd->name));
    ndSize.setHeight(fm.height());

    if (dnd->flags & NODEFLAG_DIFFERENT)
      paint->setBackgroundColor(options->differentNodeColor);
    drawText(dx, dy, nd->name);

    if (foundMark) {
      if (foundMark->addr == addr) {
	drawFoundMark(dx, dy, ndSize.width(), ndSize.height());
      }
    }
    
    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_TYPE_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawFeatvalNode(FeatvalNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  int offset;
  int nextx, nexty;
  char feat[255];

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  strcpy(feat, nd->feature);
  string_upper(feat);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, feat, dx, dy);
  }
  else {
    offset = options->offset;

    nextx = dx;
    nexty = dy;
    
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    QFontMetrics fm = paint->fontMetrics();

    nextx += fm.width(feat);
    
    if (offset < 10)
      nextx += 10;
    else
      nextx += offset;
    sz = drawGenericNode(nd->value, addr, nextx, nexty);
    nextx += sz.width();
    nexty += max(fm.height(), sz.height());

    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->setBackgroundMode(OpaqueMode);
      paint->setBackgroundColor(options->differentNodeColor);
    }

    int texty;
    if (nd->value->nodetype == NODETYPE_STRUC_NODE)
      texty = dy + (nexty - dy - fm.height()) / 2;
    else if (nd->value->nodetype == NODETYPE_REF_NODE) {
      ReentNode* reent = document->findReentrancy(((RefNode*) nd->value)->ref); 
      if (reent && (reent->value->nodetype == NODETYPE_LIST_NODE || reent->value->nodetype == NODETYPE_SET_NODE))
	texty = dy;
      else
	texty = dy + (nexty - dy - fm.height()) / 2;
    }
    else
      texty = dy;
      
    drawText(dx, texty, feat);

    if (foundMark)
      if (foundMark->addr == addr)
	drawFoundMark(dx, texty, fm.width(feat), fm.height());
  
    ndSize.setWidth(nextx - dx);
    ndSize.setHeight(nexty - dy);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_FEATVAL_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}


QSize Canvas::drawListNode(ListNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;
  QSize sz;
  int nextx;
  int nexty;
  int width;
  int height;
  int offset;
  List* structs;
  int ymiddle;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);
  
  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, "<>", dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    offset = options->offset;

    QFontMetrics fm = paint->fontMetrics();

    structs = nd->structs;

    nextx = dx;
    nexty = dy;

    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->save();
      paint->setBackgroundColor(options->differentNodeColor);
      paint->setBackgroundMode(OpaqueMode);
    }

    drawText(nextx, nexty, "<");

    if (dnd->flags & NODEFLAG_DIFFERENT)
      paint->restore();

    nextx += fm.width("<");
    height = fm.height();

    nextx += offset;

    while (structs) {
      sz = drawGenericNode(structs->item, addr, nextx, nexty, DNF_INLIST);
      height = max(height, sz.height());
      nextx += sz.width();
      if (structs->tail) {
	nextx += 2 * offset;
	drawText(nextx, nexty, ",");
	nextx += fm.width(",") + 2 * offset;
      }
      structs = structs->tail;
    }

    if (nd->tail) {
      nextx += 2 * offset;
      drawText(nextx, nexty, "|");
      nextx += fm.width("|") + 2 * offset;
      sz = drawGenericNode((GenericNode*) nd->tail, addr, nextx, nexty, DNF_INLIST);
      height = max(height, sz.height());
      nextx += sz.width();
    }

    nextx += offset;

    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->save();
      paint->setBackgroundColor(options->differentNodeColor);
      paint->setBackgroundMode(OpaqueMode);
    }

    drawText(nextx, nexty, ">");

    if (dnd->flags & NODEFLAG_DIFFERENT)
      paint->restore();

    nextx += fm.width(">");
    
    ndSize.setWidth(nextx - dx);
    ndSize.setHeight(height);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_LIST_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}
  
QSize Canvas::drawSetNode(SetNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;
  QSize sz;
  int nextx;
  int nexty;
  int width;
  int height;
  int offset;
  List* structs;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);  
  
  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, "{}", dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    offset = options->offset;

    QFontMetrics fm = paint->fontMetrics();

    structs = nd->structs;

    nextx = dx;
    nexty = dy;
  
    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->save();
      paint->setBackgroundColor(options->differentNodeColor);
      paint->setBackgroundMode(OpaqueMode);
    }

    drawText( nextx, nexty, "{");

    if (dnd->flags & NODEFLAG_DIFFERENT)
      paint->restore();
    
    nextx += fm.width("{");
    height = fm.height();

    nextx += offset;

    while (structs) {
      sz = drawGenericNode(structs->item, addr, nextx, nexty, DNF_INLIST);
      height = max(height, sz.height());
      nextx += sz.width();
      if (structs->tail) {
	nextx += 2 * offset;
	drawText(nextx, nexty, ",");
	nextx += fm.width(",") + 2 * offset;
      }
      structs = structs->tail;
    }
    
    if (nd->rest) {
      nextx += 2 * offset;
      drawText(nextx, nexty, "|");
      nextx += fm.width("|") + 2 * offset;
      sz = drawGenericNode((GenericNode*) nd->rest, addr, nextx, nexty, DNF_INLIST);
      height = max(height, sz.height());
      nextx += sz.width();
    }

    nextx += offset;

    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->save();
      paint->setBackgroundColor(options->differentNodeColor);
      paint->setBackgroundMode(OpaqueMode);
    }

    drawText(nextx, nexty, "}");

    if (dnd->flags & NODEFLAG_DIFFERENT)
      paint->restore();

    nextx += fm.width("}");

    ndSize.setWidth(nextx - dx);
    ndSize.setHeight(height);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_SET_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}
  
QSize Canvas::drawFunctionNode(FunctionNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;
  QSize sz;
  List* structs;
  int nextx;
  int nexty;
  int height;
  int offset;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, nd->functor, dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    QFontMetrics fm = paint->fontMetrics();

    offset = options->offset;

    nextx = dx;
    nexty = dy;

    QColor bgColor = paint->backgroundColor();
    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->setBackgroundColor(options->differentNodeColor);
      paint->setBackgroundMode(OpaqueMode);
    }
    
    drawText(nextx, nexty, nd->functor);
    if (foundMark)
      if (foundMark->addr == addr)
	drawFoundMark(nextx, nexty, fm.width(nd->functor), fm.height());

    paint->setBackgroundColor(bgColor);
    nextx += fm.width(nd->functor) + offset;
    height = fm.height();
    
    drawText(nextx, nexty, "(");
    nextx += fm.width("(") + offset;

    structs = nd->structs;

    while (structs) {
      sz = drawGenericNode(structs->item, addr, nextx, nexty);
      height = max(height, sz.height());
      nextx += sz.width();

      if (structs->tail) {
	drawText(nextx, nexty, ",");
	nextx += fm.width(",") + offset;
      }

      structs = structs->tail;
    }

    drawText(nextx, nexty, ")");
    nextx += fm.width(")");

    ndSize.setWidth(nextx - dx);
    ndSize.setHeight(height);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_FUNCTION_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawRelationNode(RelationNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;
  QSize sz;
  List* structs;
  int nextx;
  int nexty;
  int height;
  int offset;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, nd->functor, dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    QFontMetrics fm = paint->fontMetrics();

    offset = options->offset;

    nextx = dx;
    nexty = dy;

    QColor bgColor = paint->backgroundColor();
    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->setBackgroundColor(options->differentNodeColor); 
      paint->setBackgroundMode(OpaqueMode);
    }

    drawText(nextx, nexty, nd->functor);
    if (foundMark)
      if (foundMark->addr == addr)
	drawFoundMark(nextx, nexty, fm.width(nd->functor), fm.height());

    paint->setBackgroundColor(bgColor);
    nextx += fm.width(nd->functor) + offset;
    height = fm.height();
    
    drawText(nextx, nexty, "(");
    nextx += fm.width("(") + offset;

    structs = nd->structs;

    while (structs) {
      sz = drawGenericNode(structs->item, addr, nextx, nexty);
      height = max(height, sz.height());
      nextx += sz.width();

      if (structs->tail) {
	drawText(nextx, nexty, ",");
	nextx += fm.width(",") + offset;
      }

      structs = structs->tail;
    }

    drawText(nextx, nexty, ")");
    nextx += fm.width(")");

    ndSize.setWidth(nextx - dx);
    ndSize.setHeight(height);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_REL_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawStrucNode(StrucNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  List* featvals;
  int offset;
  int nextx;
  int nexty;
  int width = 0;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, "[]", dx, dy);
  }
  else {
    paint->save();

    offset = options->offset;

    featvals = nd->featvals;

    if (featvals)
      nextx = dx + 5;
    else
      nextx = dx;
    nexty = dy;

    sz = drawGenericNode((GenericNode*) nd->type, addr, nextx, nexty);
    nexty += sz.height();
    width = sz.width() + offset;

    while (featvals) {
      if (sz.height() > 0)
	nexty += offset;
      sz = drawGenericNode(featvals->item, addr, nextx, nexty);
      nexty += sz.height();
      width = max(sz.width(), width);
      featvals = featvals->tail;
    }

    paint->setPen(options->normalColor);

    QFontMetrics fm(options->normalFont);

    if (nd->featvals) {
      width += 2 * 5;
      if (nexty - dy < fm.height())
	nexty = dy + fm.height() + offset;

      if (nexty > QCOORD_MAX - 1)
	nexty = QCOORD_MAX - 1;
      drawLine(dx, dy, dx + 3, dy);
      drawLine(dx, dy, dx, nexty);
      drawLine(dx, nexty, dx + 3, nexty);
      
      drawLine(dx + width, dy, dx + width - 3, dy);
      drawLine(dx + width, dy, dx + width, nexty);
      drawLine(dx + width, nexty, dx + width - 3, nexty);
      
      width++;
      nexty++;
    }

    ndSize.setWidth(width);
    ndSize.setHeight(nexty - dy);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_STRUC_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawRefNode(RefNode* nd, const NodeAddress& addrParent, int dx, int dy, unsigned short flags) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  int offset;
  int nextx;
  int nexty;
  int width;
  int height;
  char buf[10];

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd  = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  paint->save();
  
  paint->setFont(options->normalFont);
  if (dnd->flags & NODEFLAG_HIDDEN)
    paint->setPen(options->hiddenColor);
  else
    paint->setPen(options->normalColor);
  
  QFontMetrics fm = paint->fontMetrics();
  
  offset = options->offset;
  
  sprintf(buf, "%d", nd->ref);
  
  width = 2 * offset + fm.width(buf);
  height = fm.height();
  
  ReentNode* reent;
  reent = document->findReentrancy(nd->ref);

  if ((dnd->flags & NODEFLAG_EXPANDED) && (!(dnd->flags & NODEFLAG_HIDDEN))) {
    if (reent) {
      nextx = dx + width + 3;
      nexty = dy;
      sz = drawGenericNode(reent->value, addr, nextx, nexty);
      width += 3 + sz.width();
      height = max(height, sz.height());
    }
  }
  
  nextx = dx + 2;
  
  if (reent && (reent->value->nodetype == NODETYPE_LIST_NODE || reent->value->nodetype == NODETYPE_SET_NODE))
    nexty = dy;
  else if (flags & DNF_INLIST)
    nexty = dy;
  else
    nexty = dy + (max(height, fm.height()) - fm.height()) / 2;
  
  if (nd->flags & NODEFLAG_STRIKEOUT) {
    paint->save();
    paint->setPen(options->markerColor);
    QPen pen = paint->pen();
    pen.setWidth(3);
    paint->setPen(pen);
    paint->drawLine(dx, nexty + fm.height(), dx + fm.width(buf) + 4, nexty);
    paint->drawLine(dx, nexty, dx + fm.width(buf) + 4, nexty + fm.height());
    paint->restore();
  }

  QColor bgColor = paint->backgroundColor();
  if (dnd->flags & NODEFLAG_DIFFERENT) {
    paint->setBackgroundMode(OpaqueMode);
    paint->setBackgroundColor(options->differentNodeColor);
  }
    
  drawText(nextx, nexty, buf);
  paint->setBackgroundColor(bgColor);
  
  nextx = dx;
  
  drawRect(nextx, nexty, 4 + fm.width(buf), fm.height());

  if (foundMark)
    if (foundMark->addr == addr)
      drawFoundMark(nextx - 1, nexty - 1, 6 + fm.width(buf), fm.height() + 2);
  
  ndSize.setWidth(width);
  ndSize.setHeight(height);
  

  paint->restore();

  if (realDraw)
    addToClick(addr, NODETYPE_REF_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));
  
  return ndSize;
}

QSize Canvas::drawAtomNode(AtomNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, nd->atom, dx, dy);
  }
  else {
    paint->save();

    paint->setFont(options->normalFont);
    paint->setPen(options->normalColor);

    QFontMetrics fm = paint->fontMetrics();

    ndSize.setWidth(fm.width(nd->atom));
    ndSize.setHeight(fm.height());

    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->setBackgroundMode(OpaqueMode);
      paint->setBackgroundColor(options->differentNodeColor);
    }
    drawText(dx, dy, nd->atom);

    if (foundMark)
      if (foundMark->addr == addr)
	drawFoundMark(dx, dy, fm.width(nd->atom), fm.height());
    
    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_TYPE_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawTreeNode(TreeNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  List* subtrees;
  int offset;
  int tx, ty;
  int liney;
  int nextx;
  int nexty;
  int tw;
  int th;
  int strucw;
  int sw;
  int sh;

  NodeAddress addr(addrParent, (GenericNode*) nd);
  dnd = dispNodes->addDisplayableNode(addr, (GenericNode*) nd);

  offset = options->offset;

  if (dnd->flags & NODEFLAG_HIDDEN) {
    ndSize = drawHiddenNode(dnd, nd->label, dx, dy);
  }
  else {
    paint->save();

    if (nd->subtrees) {
      paint->setFont(options->treeFont);
      paint->setPen(options->treeColor);
    }
    else {
      paint->setFont(options->leafFont);
      paint->setPen(options->leafColor);
    }
    QFontMetrics fm = paint->fontMetrics();

    tw = max(10 * offset, fm.width(nd->label));
    
    strucw = 0;

    if (dnd->flags & NODEFLAG_EXPANDED) {
      GenericNode* struc = document->findNodeById(nd->linkid);
      if (struc) {
	sz = getNodeDimensions(struc, addr, 0, 0);
	strucw = sz.width();
	tw = max(tw, strucw);
      }
    }

    subtrees = nd->subtrees;

    sw = 0;
    sh = 0;

    while (subtrees) {
      sz = getNodeDimensions(subtrees->item, addr, 0, 0);
      sw += sz.width() + 10 * offset;
      sh = max(sh, sz.height());
      subtrees = subtrees->tail;
    }

    tw = max(tw, sw);
    
    tx = dx + tw / 2;
    ty = dy;

      
    QColor bgColor = paint->backgroundColor();
    if (dnd->flags & NODEFLAG_DIFFERENT) {
      paint->setBackgroundMode(OpaqueMode);
      paint->setBackgroundColor(options->differentNodeColor);
    }
    
    drawText(tx - fm.width(nd->label) / 2, ty, nd->label);
    if (foundMark)
      if (foundMark->addr == addr)
	drawFoundMark(tx - fm.width(nd->label) / 2, ty, fm.width(nd->label), fm.height());

    paint->setBackgroundColor(bgColor);

    nexty = ty + fm.height();
    
    if (dnd->flags & NODEFLAG_EXPANDED) {
      GenericNode* struc = document->findNodeById(nd->linkid);
      if (struc) {
	sz = drawGenericNode(struc, addr, tx - strucw / 2, nexty);
	nexty += sz.height();
      }
    }

    liney = nexty;

    subtrees = nd->subtrees;

    nextx = tx - sw / 2;
    if (subtrees)
      nexty += 20 * offset;

    int arclabelX;
    int arclabelY;

    while (subtrees) {
      nextx += 5 * offset;
      sz = drawGenericNode(subtrees->item, addr, nextx, nexty);
      //      paint->save();
      paint->setPen(options->treeLineColor);
      drawLine(tx, liney, nextx + sz.width() / 2, nexty);

      if (((TreeNode*) subtrees->item)->arclabel != NULL) {
	TreeNode* tn = (TreeNode*) subtrees->item;
	paint->setPen(options->arclabelColor);
	paint->setFont(options->arclabelFont);
	fm = paint->fontMetrics();
	int labelWidth = fm.width(tn->arclabel);
	
	arclabelY = liney + (nexty - liney - fm.height()) / 2;
	arclabelX = tx + (nextx + sz.width() / 2 - tx) / 2;
	if (arclabelX < tx)
	  arclabelX -= 3 * labelWidth / 4;
	else if (arclabelX > tx)
	  arclabelX -= labelWidth / 4;
	else
	  arclabelX -= labelWidth / 2;
	if (realDraw)
	  paint->fillRect(arclabelX - ARCLABEL_FRAME_OFFS_X, arclabelY - ARCLABEL_FRAME_OFFS_Y, labelWidth + 2 * ARCLABEL_FRAME_OFFS_X, fm.height() + 2 * ARCLABEL_FRAME_OFFS_Y, options->arclabelBgColor);
	
	drawRect(arclabelX - ARCLABEL_FRAME_OFFS_X, arclabelY - ARCLABEL_FRAME_OFFS_Y, labelWidth + 2 * ARCLABEL_FRAME_OFFS_X, fm.height() + 2 * ARCLABEL_FRAME_OFFS_Y);
	drawText(arclabelX, arclabelY, tn->arclabel, strlen(tn->arclabel));
      }
	

      //paint->restore();
      nextx += sz.width() + 5 * offset;
      subtrees = subtrees->tail;
    }

    ndSize.setWidth(tw);
    ndSize.setHeight(nexty + sh - dy);

    paint->restore();
  }

  if (realDraw)
    addToClick(addr, NODETYPE_TREE_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}

QSize Canvas::drawTailNode(TailNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  return drawGenericNode((GenericNode*) nd->tail, addrParent, dx, dy);
}

QSize Canvas::drawRestNode(RestNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  return drawGenericNode((GenericNode*) nd->rest, addrParent, dx, dy);
}


QSize Canvas::drawDisjunctionNode(DisjunctionNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  int offset;
  int nextx;
  int nexty;
  int width;
  int height;
  List* operands;

  NodeAddress addr(addrParent, nd);
  dnd = dispNodes->addDisplayableNode(addr, nd);

  // Disjunction operators can't be hidden, so no hiding code here

  paint->save();

  paint->setFont(options->normalFont);
  paint->setPen(options->normalColor);

  offset = options->offset;

  QFontMetrics fm = paint->fontMetrics();

  nextx = dx;
  nexty = dy;

  width = 0;
  height = fm.height();

  operands = nd->operands;

  int opHeight = fm.ascent();
  int opWidth = opHeight;
  if (opWidth % 2 != 0)   // make sure operator is symmetric
    opWidth++;

  while (operands != NULL) {
    sz = drawGenericNode(operands->item, addr, nextx, nexty);

    if ((dnd->flags & NODEFLAG_MULTILINE) == 0) {
      height = max(height, sz.height());
      nextx += sz.width();
      if (operands->tail != NULL) {
	nextx += 2 * offset;
	if (dnd->flags & NODEFLAG_DIFFERENT) {
	  if (realDraw)
	    paint->fillRect(nextx, nexty, nextx + opWidth, nexty + opHeight, options->differentNodeColor);
	}
	drawOrOperator(nextx, opWidth, nexty, nexty + opHeight);
	nextx += opWidth + 2 * offset;
      }
      width = nextx - dx;
    }
    else {
      nextx = dx;
      if (operands->tail != NULL) {
	nexty += sz.height() + offset;
	if (dnd->flags & NODEFLAG_DIFFERENT) {
	  if (realDraw)
	    paint->fillRect(nextx, nexty, nextx + opWidth, nexty + opHeight, options->differentNodeColor);
	}
	drawOrOperator(nextx, opWidth, nexty, nexty + opHeight);
	nextx += opWidth + 2 * offset;
	width = max(width, sz.width() + fm.width("V") + 2 * offset);
      }
      else {
	nexty += sz.height();
	width = max(width, sz.width());
      }
      height = nexty - dy;
    }

    operands = operands->tail;
  }
  
  ndSize.setWidth(width);
  ndSize.setHeight(height);

  paint->restore();

  if (realDraw)
    addToClick(addr, NODETYPE_DISJUNCTION_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}


QSize Canvas::drawConjunctionNode(ConjunctionNode* nd, const NodeAddress& addrParent, int dx, int dy) {
  DisplayableNode* dnd;
  QSize ndSize(0, 0);
  QSize sz;
  int offset;
  int nextx;
  int nexty;
  int width;
  int height;
  List* operands;

  NodeAddress addr(addrParent, nd);
  dnd = dispNodes->addDisplayableNode(addr, nd);

  // Conjunction operators can't be hidden, so no hiding code here

  paint->save();

  paint->setFont(options->normalFont);
  paint->setPen(options->normalColor);

  offset = options->offset;

  QFontMetrics fm = paint->fontMetrics();

  nextx = dx;
  nexty = dy;

  width = 0;
  height = fm.height();

  operands = nd->operands;

  int opHeight = fm.ascent();
  int opWidth = opHeight;
  if (opWidth % 2 != 0)   // make sure operator is symmetric
    opWidth++;
	
  while (operands != NULL) {
    sz = drawGenericNode(operands->item, addr, nextx, nexty);

    if ((dnd->flags & NODEFLAG_MULTILINE) == 0) {
      height = max(height, sz.height());
      nextx += sz.width();
      if (operands->tail != NULL) {
	nextx += 2 * offset;
	if (dnd->flags & NODEFLAG_DIFFERENT) {
	  if (realDraw)
	    paint->fillRect(nextx, nexty, nextx + opWidth, nexty + opHeight, options->differentNodeColor);
	}
	drawAndOperator(nextx, opWidth, nexty, nexty + opHeight);
	nextx += opWidth + 2 * offset;
      }
      width = nextx - dx;
    }
    else {
      nextx = dx;
      if (operands->tail != NULL) {
	nexty += sz.height() + offset;
	if (dnd->flags & NODEFLAG_DIFFERENT) {
	  if (realDraw)
	    paint->fillRect(nextx, nexty, nextx + opWidth, nexty + opHeight, options->differentNodeColor);
	}
	drawAndOperator(nextx, opWidth, nexty, nexty + opHeight);
	nextx += opWidth + 2 * offset;
	width = max(width, sz.width() + fm.width("V") + 2 * offset);
      }
      else {
	nexty += sz.height();
	width = max(width, sz.width());
      }
      height = nexty - dy;
    }

    operands = operands->tail;
  }
  
  ndSize.setWidth(width);
  ndSize.setHeight(height);

  paint->restore();

  if (realDraw)
    addToClick(addr, NODETYPE_CONJUNCTION_NODE, QRect(dx, dy, ndSize.width(), ndSize.height()));

  return ndSize;
}
