/*  $Id: gspacingdialog.cpp,v 1.5 2003/05/22 16:43:12 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Implementation of the structure spacing dialog.

    @author Holger Wunsch */

#include <qvariant.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "gspacingdialog.h"
#include "gspacingdialog.moc"

/* 
 *  Constructs a GSpacingDialog as a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
GSpacingDialog::GSpacingDialog(Options* options, QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    if ( !name )
	setName( "GSpacingDialog" );

    okButton = new QPushButton( this, "okButton" );
    okButton->setGeometry( QRect( 40, 70, 80, 27 ) );
    okButton->setDefault( TRUE );
    connect(okButton, SIGNAL(clicked(void)), SLOT(okClicked(void)));

    spinBox1 = new QSpinBox( this, "spinBox1" );
    spinBox1->setGeometry( QRect( 140, 20, 80, 20 ) );
    spinBox1->setValue(options->offset);
    spinBox1->setFocus();

    textLabel2 = new QLabel( this, "textLabel2" );
    textLabel2->setGeometry( QRect( 20, 20, 103, 20 ) );

    cancelButton = new QPushButton( this, "cancelButton" );
    cancelButton->setGeometry( QRect( 140, 70, 80, 27 ) );
    connect(cancelButton, SIGNAL(clicked(void)), SLOT(cancelClicked(void)));
    languageChange();
    resize( QSize(244, 117).expandedTo(minimumSizeHint()) );

    // signals and slots connections

    // buddies
    textLabel2->setBuddy( spinBox1 );

    this->options = options;
}

/*
 *  Destroys the object and frees any allocated resources
 */
GSpacingDialog::~GSpacingDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void GSpacingDialog::languageChange()
{
    setCaption( tr( "Structure Spacing" ) );
    okButton->setText( tr( "OK" ) );
    textLabel2->setText( tr( "&Structure Spacing:" ) );
    cancelButton->setText( tr( "Cancel" ) );
}


/** Called when the user clicked OK. Updates the spacing value in the associated
    options object. */
void GSpacingDialog::okClicked(void) {
  options->offset = spinBox1->value();
  emit accept();
}


/** Called when the user clicked Cancel. */
void GSpacingDialog::cancelClicked(void) {
  emit reject();
}
