/*  $Id: mainw.cpp,v 1.38 2003/05/24 01:55:02 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of class MainWindow.

    @author Holger Wunsch */

#include <stdio.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include "mainw.h"
#include "config.h"
#include "global.h"
#include "vcanvas.h"

#define Inherited QDialog

#include <kstatusbar.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kmenubar.h>
#include <kiconloader.h>
#include <kaboutkde.h>
#include <kicontheme.h>
#include <kfiledialog.h>
#include <kmessagebox.h>

#include <qfiledialog.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qkeycode.h>
#include <qaccel.h>

#include "node.h"

#include "mainw.moc"

#include "compare.h"

#include "dataparser.h"
#include "gdataexchange.h"
#include "gspacingdialog.h"

#include "tools.h"


/** <code>true</code> if the user changed application options. */
extern bool optionChanged;


MainWindow::MainWindow(QWidget * parent, const char *name)
  : KMainWindow(parent, name)
{
  QPixmap pix;
  KPixmap kpix;
  QImage image;
  QIconSet icon;

  KIconLoader* loader = KApplication::kApplication()->iconLoader();

  QString help(APP_NAME);
  help += " ";
  help += GRISU_VERSION;
  
  QAccel* accel = new QAccel(this);

  setPlainCaption(help);

  resize(400, 300);
  mainMenu = menuBar();

  // File Menu
  fileMenu = new QPopupMenu;
  fileMenu->setCheckable(true);
  // & Fr unterstrichenen Buchstaben ...

  //###########################################
  // FileMenu
  
  pix = loader->loadIcon("fileopen", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  fileMenu->insertItem (icon, "&Load Data...", this,
			SLOT( slotLoad() ), ALT + Key_L);  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscalePixmap(icon, fileMenu->paletteBackgroundColor());
#endif
  fileMenu->insertItem (icon, "&Save Data...", this,
			SLOT( slotSave() ), ALT + Key_S, MID_FILE_SAVE );
  fileMenu->setItemEnabled(MID_FILE_SAVE, false);
  fileMenu->insertSeparator();
  fileMenu->insertItem ("E&xit Grisu", this,
			SLOT( close() ), ALT + Key_X );

  // Additional accelerators for the file menu
  accel->connectItem(accel->insertItem(ALT + Key_Q),
		     this,
		     SLOT(close()));
  
  //###########################################
  //  Data Menu
  dataMenu = new QPopupMenu;
  dataMenu->setCheckable(true);

  pix = loader->loadIcon("showtree", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscalePixmap(icon, dataMenu->paletteBackgroundColor());
#endif
  dataMenu->insertItem (icon, "Show &Tree",
			this,
			SLOT( showTreeSlot()), CTRL + Key_T, MID_DATA_SHOW_TREE );
  dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, false);
  pix = loader->loadIcon("showstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscalePixmap(icon, dataMenu->paletteBackgroundColor());
#endif
  dataMenu->insertItem (icon, "Show &Structure",
			this,
			SLOT( showStructureSlot()), CTRL + Key_S, MID_DATA_SHOW_STRUC);
  dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, false);
  /*  dataMenu->insertSeparator();
      dataMenu->insertItem ("&Compare two structures",
      this,
      SLOT(compareStructuresSlot()), ALT + Key_C, MID_DATA_COMPARE_STRUC);
      dataMenu->setItemEnabled(MID_DATA_COMPARE_STRUC, false);
      dataMenu->insertItem ("Compare two &trees",
      this,
      SLOT(compareTreesSlot()), ALT + Key_T, MID_DATA_COMPARE_TREE );
      dataMenu->setItemEnabled(MID_DATA_COMPARE_TREE, false);*/
  dataMenu->insertSeparator();
  pix = loader->loadIcon("deletedata", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscalePixmap(icon, dataMenu->paletteBackgroundColor());
#endif
  dataMenu->insertItem (icon, "&Delete Data",
			this,
			SLOT( deleteDataSlot()), CTRL + Key_D, MID_DATA_DELETE_DATA );
  dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, false);
  dataMenu->insertSeparator();
  dataMenu->insertItem("Delete &All Data",
		       this,
		       SLOT(deleteAllDataSlot()), SHIFT + CTRL + Key_D, MID_DATA_DELETE_ALL_DATA);
  dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, false);

  optionMenu = new QPopupMenu;

  
  colorMenu = new QPopupMenu;
  colorMenu->setCheckable(true);
  colorMenu->insertItem("&Attributes...", this, 
			SLOT(changeNormalColorSlot()));
  colorMenu->insertItem("&Types...", this,
			SLOT(changeTypeColorSlot()));
  colorMenu->insertItem("Te&xt...", this,
			SLOT(changeTextColorSlot()));
  colorMenu->insertItem("Inner T&ree Nodes...", this,
			SLOT(changeTreeColorSlot()));
  colorMenu->insertItem("&Edges...", this,
			SLOT(changeTreeLineColorSlot()));
  colorMenu->insertItem("Lea&ves...", this,
			SLOT(changeLeafColorSlot()));
  colorMenu->insertItem("Edge L&abels...", this,
			SLOT(changeArcLabelColorSlot()));
  colorMenu->insertItem("Edge Label &Background...", this,
			SLOT(changeArcLabelBgColorSlot()));
  colorMenu->insertItem("&Hidden Nodes...", this,
			SLOT(changeHiddenColorSlot()));
  colorMenu->insertItem("Highlighted &Structures...", this,
			SLOT(changeMarkerColorSlot()));
  colorMenu->insertItem("&Found Structures...", this,
			SLOT(changeFoundColorSlot()));
  colorMenu->insertItem("&Differing Structures...", this,
			SLOT(changeDifferentNodesColorSlot()));
  colorMenu->insertItem("Err&ors...", this, 
			SLOT(changeErrorColorSlot()));
  /*  colorMenu->insertSeparator();
      colorMenu->insertItem("&Background color", this, 
      SLOT(changeBackgroundColorSlot()));*/
  
  QObject::connect(this,
	  SIGNAL(changeColorSignal(int, QWidget*)),
	  globalOptions, 
	  SLOT(changeColorSlot(int, QWidget*)));
  
  fontMenu = new QPopupMenu;
  fontMenu->setCheckable(true);
  fontMenu->insertItem("&Attributes...", this, 
		       SLOT(changeNormalFontSlot()));
  fontMenu->insertItem("&Types...", this, 
		       SLOT(changeTypeFontSlot()));
  fontMenu->insertItem("Inner T&ree Nodes...", this, 
		       SLOT(changeTreeFontSlot()));
  fontMenu->insertItem("&Leaves...", this, 
		       SLOT(changeLeafFontSlot()));
  fontMenu->insertItem("&Edge L&abels...", this, 
		       SLOT(changeArcLabelFontSlot()));
  fontMenu->insertItem("Te&xt...", this, 
		       SLOT(changeTextFontSlot()));
  fontMenu->insertItem("&Errors...", this, 
		       SLOT(changeErrorFontSlot()));
  
  QObject::connect(this,
	  SIGNAL(changeFontSignal(int)),
	  globalOptions, 
	  SLOT(changeFontSlot(int)));
  
  optionMenu->insertItem("Auto-&Expand Tags", this, SLOT(toggleAutoExpandSlot()), 0, MID_OPTIONS_EXPAND_REENT);
  optionMenu->setItemChecked(MID_OPTIONS_EXPAND_REENT, globalOptions->expandReentrancies);
  optionMenu->insertItem("&Windows Fit Structure Size", this, SLOT(toggleFitSlot()), 0, MID_OPTIONS_FIT);
  optionMenu->setItemChecked(MID_OPTIONS_FIT, globalOptions->fitToDrawing);
  optionMenu->insertItem("&Highlight Structures", this, SLOT(toggleHighlightSlot()), 0, MID_OPTIONS_HIGHLIGHT);
  optionMenu->setItemChecked(MID_OPTIONS_HIGHLIGHT, globalOptions->highlightStructures);
  optionMenu->insertItem("&Automatically Open Windows", this, SLOT(toggleAutoOpenSlot()), 0, MID_OPTIONS_AUTOOPEN);
  optionMenu->setItemChecked(MID_OPTIONS_AUTOOPEN, globalOptions->autoOpenWindows);
  optionMenu->insertSeparator();
  optionMenu->insertItem("&Structure Spacing...", this,
			  SLOT(optionsSlot()));
  optionMenu->insertSeparator();
  optionMenu->insertItem("&Colors", colorMenu);
  optionMenu->insertItem("&Fonts", fontMenu);

  windowMenu = new QPopupMenu();

  pix = loader->loadIcon("closeviews", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscalePixmap(icon, windowMenu->paletteBackgroundColor());
#endif
  windowMenu->insertItem(icon, "&Close All Data Windows", this, SLOT(closeAllSlot()), 0, MID_WINDOW_CLOSEALL);
  windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, false);

  mainMenu->insertItem("&File", fileMenu);
  mainMenu->insertItem("&Data", dataMenu);
  mainMenu->insertItem("&Options", optionMenu);
  mainMenu->insertItem("&Window", windowMenu);

  helpMenu = new QPopupMenu();
  pix = loader->loadIcon("Grisu", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  helpMenu->insertItem(icon, "About &Grisu...", this, SLOT(aboutGrisuSlot()), 0, MID_ABOUT_GRISU);
  helpMenu->insertItem("About &KDE...", this, SLOT(aboutKDESlot()), 0, MID_ABOUT_KDE);

  mainMenu->insertItem("&Info", helpMenu);


  /*---------------------------------
    Status Bar
    -------------------------------*/

  statusBar = KMainWindow::statusBar();

  statusBar->insertItem("", SID_CONNECTION_STATE, 1);
  statusBar->setItemAlignment(SID_CONNECTION_STATE, Qt::AlignLeft);
  statusBar->insertItem("", SID_CONNECTION_HOST);


  /***********************/
  /* And now the toolbar */
  /***********************/

  // Create toolbar...
  mainToolBar = toolBar();

  // Add toolbar buttons
  mainToolBar->setTitle("Grisu Main Toolbar");

  pix = loader->loadIcon("fileopen", KIcon::User);
  mainToolBar->insertButton(pix, TID_FILE_OPEN, SIGNAL(clicked()), this, SLOT(slotLoad()),
			TRUE, "Load Data");
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  mainToolBar->insertButton(pix, TID_FILE_SAVE, SIGNAL(clicked()), this, SLOT(slotSave()),
			TRUE, "Save Data");
  mainToolBar->setItemEnabled(TID_FILE_SAVE, false);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscaleToolbarButton(mainToolBar, pix, TID_FILE_SAVE);
#endif

#if (KDE_VERSION >= 210)
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);
#endif

  pix = loader->loadIcon("showtree", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_SHOW_TREE, SIGNAL(clicked()), this, SLOT(showTreeSlot()),
			TRUE, "Show Tree");

#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscaleToolbarButton(mainToolBar, pix, TID_DATA_SHOW_TREE);  
#endif
  mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, false);
  

  pix = loader->loadIcon("showstruc", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_SHOW_STRUC, SIGNAL(clicked()), this, SLOT(showStructureSlot()),
			TRUE, "Show Structure");
  mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, false);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscaleToolbarButton(mainToolBar, pix, TID_DATA_SHOW_STRUC);
#endif

#if (KDE_VERSION >=210)
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);
#endif

  pix = loader->loadIcon("deletedata", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_DELETE_DATA, SIGNAL(clicked()), this, SLOT(deleteDataSlot()),
			TRUE, "Delete Data");
  mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, false);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscaleToolbarButton(mainToolBar, pix, TID_DATA_DELETE_DATA);
#endif

#if (KDE_VERSION >=210)
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);
#endif

  pix = loader->loadIcon("closeviews", KIcon::User);
  mainToolBar->insertButton(pix, TID_WINDOW_CLOSEALL, SIGNAL(clicked()), this, SLOT(closeAllSlot()),
			TRUE, "Close All Data Windows");
  mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, false);
#if (KDE_VERSION >= 210 && QT_VERSION >= 0x030000)
  setGrayscaleToolbarButton(mainToolBar, pix, TID_WINDOW_CLOSEALL);
#endif


  listBox = new QListBox(this);
  listBox->setAutoScrollBar(TRUE);
  listBox->setSelectionMode(QListBox::Extended);
  QObject::connect(listBox, SIGNAL(selectionChanged()), this, SLOT(listSelectionChangedSlot()));

  KMainWindow::statusBar()->show();  

  setCentralWidget(listBox);
  QObject::connect(listBox,
		   SIGNAL(selected(int)),
		   this,
		   SLOT(doubleKlickSlot(int)));

  setMinimumSize(100,100);
  setMaximumSize(9999,9999);

  views = new QList<ViewCanvas>;

  closingAll = false;

  QObject::connect(this, SIGNAL(optionChangedSignal()), this, SLOT(optionChangedSlot()));
}


MainWindow::~MainWindow() {
  closeAllSlot();

  //delete mainToolBar;
  //delete menuBar;
  //delete listBox;

  if (views)
    delete views;
}


void MainWindow::slotSave() {
  QString Filename;
  bool ok = false;

  int numSel = 0;
  for (int i = 0; i < listBox->count(); i++) 
    if (listBox->isSelected(i))
      numSel++;
  
  while (!ok) {
    if (globalOptions->useKFileDialog)
      Filename = KFileDialog::getSaveFileName(NULL, "*.grisu|Grisu data files (*.grisu)\n*|All files (*)", this, "Save Data");
    else
      Filename = QFileDialog::getSaveFileName(NULL, "Grisu data files (*.grisu);;All files (*)", this, NULL, "Save Data");
    if (Filename.length() > 0) {
      if (QFile::exists(Filename) && numSel == 1) {
	char buf[500];
	sprintf(buf, "The file %s already exists.\nOverwrite?", (const char*) Filename.local8Bit());
	switch (QMessageBox::information(this, "Save", buf, "&Yes", "&No", "Cancel", 0, 2)) {
	case 0:
	  ok = true;
	  break;
	case 1:
	  break;
	case 2:
	  return;
	}
      }
      else
	ok = true;
    }
    else if (Filename.isEmpty())
      return;
  }
  
  if (Filename.length() > 0) {
    QString FullFilename;

    for (int i = 0; i < listBox->count(); i++) {
      if (listBox->isSelected(i)) {
	if (numSel > 1) {
	  char buf[500];
	  sprintf(buf, "%s.%d.grisu", (const char*) Filename.local8Bit(), i + 1);
	  FullFilename = buf;
	}
	else
	  FullFilename = Filename;

	FILE* file = fopen((const char*) FullFilename.local8Bit(), "w");
	
	if (file) {
	  setCursor(waitCursor);
	  
	  Document* d = documents.at(listBox->currentItem());
	  d->write(file);
	  fprintf(file, "\n");
	  fclose(file);
	  
	  setCursor(arrowCursor);
	}
	else {
	  char buf[500];
	  sprintf(buf, "Could not open file %s", (const char*) FullFilename.local8Bit());
	  QMessageBox::critical(this, "Save", buf);
	}
      }
    }
  } 
}

/** Loads a GRISU data file and creates a new view window, depending on the user-settings.
    @param fileName the name of the data file. */
void MainWindow::loadDataFile(const char* fileName) {
  FILE* file = fopen(fileName, "r");
  
  if (file) {
    setCursor(waitCursor);
    char* line = new char[250000];
    Document* newDoc;
    
    fgets(line, 250000, file);
    
    newDoc = DataParser().parse(line);
    if (newDoc) {
      documents.append(newDoc);
      updateListBox();
      if (globalOptions->autoOpenWindows) {
	if (newDoc->showDefault != 0)
	  newViewWindow(newDoc, newDoc->showDefault);
	else {
	  if (newDoc->tree)
	    newViewWindow(newDoc, DRAW_TREE);
	  else
	    newViewWindow(newDoc, DRAW_STRUC);
	}
      }
      dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, true);
    }
    delete [] line;
    
    fclose(file);
    
    setCursor(arrowCursor);
  }
  else {
    char buf[500];
    sprintf(buf, "Could not open file %s", fileName);
    QMessageBox::critical(this, "Save", buf);
  }
}

void MainWindow::slotLoad() {
  if (globalOptions->useKFileDialog) {
    KFileDialog fileDlg(NULL, "*.grisu|Grisu data files (*.grisu)\n*|All files (*)", this, "openfile", true);
    fileDlg.setCaption("Load Data"); 
    fileDlg.setMode(KFile::Files);
    fileDlg.setIcon(application->icon());
    fileDlg.exec();
    QStringList Filenames = fileDlg.selectedFiles();
    for (int i = 0; i < Filenames.count(); i++)
      loadDataFile((const char*) Filenames[i]);
  }
  else {
    QFileDialog fileDlg("*.grisu", "Grisu data files (*.grisu);;All files (*)", this, "openfile", true);
    fileDlg.setCaption("Load Data"); 
    fileDlg.setMode(QFileDialog::ExistingFiles);
    fileDlg.setIcon(application->icon());
    fileDlg.exec();
    QStringList Filenames = fileDlg.selectedFiles();
    for (int i = 0; i < Filenames.count(); i++)
      loadDataFile((const char*) Filenames[i]);
  }
}

void MainWindow::updateListBox() {
  listBox->clear();
  Document* iter;
  int i;
  for(iter=documents.first();
      !(iter==NULL);
      iter=documents.next()) {
    QString help=(iter->getName()).copy();
    listBox->insertItem(help);
  }
  listSelectionChangedSlot();
}



ViewCanvas* MainWindow::newViewWindow(Document* d, int what, int x, int y, int w, int h) {
  ViewCanvas *ca=new ViewCanvas(d, what, globalOptions, this);
  if (x != -1 && y != -1)
    ca->move(x, y);
  ca->resize(w, h);
  ca->show();
  ca->resizeEvent(NULL);
  ca->redraw();

  views->append(ca);

  QObject::connect(ca, SIGNAL(closing(ViewCanvas*)), this, SLOT(viewIsClosing(ViewCanvas*)));

  windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, true);
  mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, true);

  return ca;
}


void MainWindow::doubleKlickSlot(int index) {
  Document* doc = documents.at(index);
  if (doc->showDefault != 0)
    newViewWindow(doc, doc->showDefault);
  else {
    if (doc->tree)
      newViewWindow(doc, DRAW_TREE);
    else
      newViewWindow(doc, DRAW_STRUC);
  }
}


void MainWindow::showStructureSlot() {
  for (int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i))
      newViewWindow(documents.at(i), DRAW_STRUC);
  }
}
    
void MainWindow::showTreeSlot() {
  for (int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i))
      newViewWindow(documents.at(i), DRAW_TREE);
  }    
}

void MainWindow::deleteData(bool selOnly) {
  QList<ViewCanvas> delvc;
  ViewCanvas* vc;
  int j = 0;
  for (int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i) || !selOnly) {
      vc = views->first();
      while (vc) {
	if (vc->showsDoc(documents.at(i - j)))
	  delvc.append(vc);
	vc = views->next();
      }
      documents.remove(i - j);
      j++;
    }
  }

  updateListBox();

  vc = delvc.first();
  while (vc) {
    vc->close();
    vc = delvc.next();
  }

  if (documents.count() == 0)
    dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, false);
}
  

void MainWindow::deleteDataSlot() {
  deleteData(true);
}


void MainWindow::deleteAllDataSlot(void) {
  deleteData(false);
}


void MainWindow::closeAllSlot() {
  closingAll = true;

  ViewCanvas* vc = views->first();

  while (vc) {
    vc->close();
    views->remove();
    vc = views->current();
  }

  views->clear();

  closingAll = false;
}

void MainWindow::viewIsClosing(ViewCanvas* view) {
  if (!closingAll) {
    ViewCanvas* vc = views->first();
    while (vc) {
      if (vc == view) {
	views->remove();
	if (views->count() > 0) {
	  windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, true);
	  mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, true);
	}
	else {
	  windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, false);
	  mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, false);
	}
	return;
      }
      vc = views->next();
    }
  }
  else {
    windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, false);
    mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, false);
  }
}     


void MainWindow::processData(GBuffer* buffer) {
  char* line;
  Document* newDoc;

  while ((line = (char*) buffer->readLine()) != NULL) {
    
    // Dump all incoming data to a file if requested
    if (dumpData) {
      FILE* dumpFile = fopen(DATADUMP_FILENAME, "a");
      fputs(line, dumpFile);
      fclose(dumpFile);
    }

    // Special command: quit Grisu
    if (strncmp(line, "!quit", 5) == 0) {
      delete [] line;
      close();
      return;
    }
      
    // Parse data package and display it
    newDoc = DataParser().parse(line);
    if (newDoc) {
      documents.append(newDoc);
      updateListBox();
      if (globalOptions->autoOpenWindows) {
	if (newDoc->showDefault != 0) 
	  newViewWindow(newDoc, newDoc->showDefault);
	else {
	  if (newDoc->tree)
	    newViewWindow(newDoc, DRAW_TREE);
	  else
	    newViewWindow(newDoc, DRAW_STRUC);
	}
      }
      dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, true);
    }
    delete [] line;
  }
}
  

void MainWindow::optionsSlot() {
  GSpacingDialog* dlg = new GSpacingDialog(globalOptions, this, NULL, true);
  dlg->setIcon(application->icon());
  if (dlg->exec() == QDialog::Accepted)
    optionChanged = true;
}

void MainWindow::compareStructuresOrTrees(int what) {
  ViewCanvas* vcanvas0;
  ViewCanvas* vcanvas1;
  StructureCompare* sc;

  int index1 = -1;
  int index2 = -1;
  for (int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i)) {
      if (index1 == -1)
	index1 = i;
      else {
	index2 = i;
	break;
      }
    }
  }
	
  if (what == DRAW_STRUC) {
    vcanvas0 = newViewWindow(documents.at(index1), DRAW_STRUC);
    vcanvas1 = newViewWindow(documents.at(index2), DRAW_STRUC);
    sc = new StructureCompare(documents.at(index1), NodeAddress(), documents.at(index1)->data->at(0), vcanvas0->canvas->getDispNodes(), documents.at(index2), NodeAddress(), documents.at(index2)->data->at(0), vcanvas1->canvas->getDispNodes());
  }
  else if (what == DRAW_TREE) {
    vcanvas0 = newViewWindow(documents.at(index1), DRAW_TREE);
    vcanvas1 = newViewWindow(documents.at(index2), DRAW_TREE);
    sc = new StructureCompare(documents.at(index1), NodeAddress(), (GenericNode*) documents.at(index1)->tree, vcanvas0->canvas->getDispNodes(), documents.at(index2), NodeAddress(), (GenericNode*) documents.at(index2)->tree, vcanvas1->canvas->getDispNodes());
  }
  sc->compare();
  vcanvas0->repaint();
  vcanvas1->repaint();
  char cyclbuf[255];
  if (sc->cyclicPaths())
    sprintf(cyclbuf, " - Warning: Cyclic structures");
  else
    sprintf(cyclbuf, "");
  char buf[255];
  if (sc->differences() > 0)
    sprintf(buf, "%u difference(s)%s", sc->differences(), cyclbuf);
  else
    sprintf(buf, "No differences%s", cyclbuf);
  vcanvas0->statusBar->message(buf);
  vcanvas1->statusBar->message(buf);
}

void MainWindow::compareStructuresSlot() {
  compareStructuresOrTrees(DRAW_STRUC);
}


void MainWindow::compareTreesSlot() {
  compareStructuresOrTrees(DRAW_TREE);
}  

void MainWindow::toggleAutoExpandSlot(void) {
  globalOptions->expandReentrancies = !globalOptions->expandReentrancies;
  optionMenu->setItemChecked(MID_OPTIONS_EXPAND_REENT, globalOptions->expandReentrancies);
  optionChanged = true;
}

void MainWindow::toggleFitSlot(void) {
  globalOptions->fitToDrawing = !globalOptions->fitToDrawing;
  optionMenu->setItemChecked(MID_OPTIONS_FIT, globalOptions->fitToDrawing);
  optionChanged = true;
}

void MainWindow::toggleHighlightSlot(void) {
  globalOptions->highlightStructures = !globalOptions->highlightStructures;
  optionMenu->setItemChecked(MID_OPTIONS_HIGHLIGHT, globalOptions->highlightStructures);
  optionChanged = true;
}

void MainWindow::toggleAutoOpenSlot(void) {
  globalOptions->autoOpenWindows = !globalOptions->autoOpenWindows;
  optionMenu->setItemChecked(MID_OPTIONS_AUTOOPEN, globalOptions->autoOpenWindows);
  optionChanged = true;
}

void MainWindow::changeNormalColorSlot() {
  emit changeColorSignal(Options::NormalColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTypeColorSlot() {
  emit changeColorSignal(Options::TypeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeErrorColorSlot() {
  emit changeColorSignal(Options::ErrorColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeBackgroundColorSlot() {
  emit changeColorSignal(Options::BackgroundColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTextColorSlot() {
  emit changeColorSignal(Options::TextColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTreeColorSlot() {
  emit changeColorSignal(Options::TreeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTreeLineColorSlot() {
  emit changeColorSignal(Options::TreeLineColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeLeafColorSlot() {
  emit changeColorSignal(Options::LeafColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelColorSlot() {
  emit changeColorSignal(Options::ArcLabelColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelBgColorSlot() {
  emit changeColorSignal(Options::ArcLabelBgColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeHiddenColorSlot() {
  emit changeColorSignal(Options::HiddenColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeMarkerColorSlot() {
  emit changeColorSignal(Options::MarkerColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeFoundColorSlot() {
  emit changeColorSignal(Options::FoundColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeDifferentNodesColorSlot() {
  emit changeColorSignal(Options::DifferentNodeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeNormalFontSlot() {
  emit changeFontSignal(Options::NormalFont);
  emit optionChangedSignal();
}

void MainWindow::changeTypeFontSlot() {
  emit changeFontSignal(Options::TypeFont);
  emit optionChangedSignal();
}

void MainWindow::changeErrorFontSlot() {
  emit changeFontSignal(Options::ErrorFont);
  emit optionChangedSignal();
}

void MainWindow::changeTextFontSlot() {
  emit changeFontSignal(Options::TextFont);
  emit optionChangedSignal();
}

void MainWindow::changeTreeFontSlot() {
  emit changeFontSignal(Options::TreeFont);
  emit optionChangedSignal();
}

void MainWindow::changeLeafFontSlot() {
  emit changeFontSignal(Options::LeafFont);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelFontSlot() {
  emit changeFontSignal(Options::ArcLabelFont);
  emit optionChangedSignal();
}  

void MainWindow::optionChangedSlot() {
  ViewCanvas* vc = views->first();
  while (vc) {
    vc->canvas->updateOptions(*globalOptions);
    vc->redrawSlot();
    vc = views->next();
  }
}

void MainWindow::listSelectionChangedSlot() {
  int marked = 0;
  for (int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i))
      marked++;
  }
  
  if (marked > 0) {
    fileMenu->setItemEnabled(MID_FILE_SAVE, true);
    mainToolBar->setItemEnabled(TID_FILE_SAVE, true);

    mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, true);
    dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, true);

    dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, true);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, true);

    dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, true);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, true);
    /*    if (marked == 2) {
	  dataMenu->setItemEnabled(MID_DATA_COMPARE_STRUC, true);
	  dataMenu->setItemEnabled(MID_DATA_COMPARE_TREE, true);
	  }
	  else {
	  dataMenu->setItemEnabled(MID_DATA_COMPARE_STRUC, false);
	  dataMenu->setItemEnabled(MID_DATA_COMPARE_TREE, false);
	  }*/
  }
  else {
    fileMenu->setItemEnabled(MID_FILE_SAVE, false);
    mainToolBar->setItemEnabled(TID_FILE_SAVE, false);

    dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, false);
    mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, false);

    dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, false);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, false);

    dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, false);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, false);

    /*    dataMenu->setItemEnabled(MID_DATA_COMPARE_STRUC, false);
	  dataMenu->setItemEnabled(MID_DATA_COMPARE_TREE, false);*/
  }
}
  
void MainWindow::closeEvent(QCloseEvent* ev) {
  KMainWindow::closeEvent(ev);
  KApplication::kApplication()->quit();
}

void MainWindow::aboutGrisuSlot(void) {
  char buf[500];
  sprintf(buf, GRISU_ABOUT_STRING, GRISU_VERSION);
  KMessageBox::about(this, buf, "About GRISU");
}


void MainWindow::aboutKDESlot(void) {
  KAboutKDE aboutKDE(this);
  aboutKDE.exec();
}


void MainWindow::previousStructureSlot(ViewCanvas* replace) {
  Document* doc = replace->canvas->getDocument();
  int index = -1;
  int showMode;

  for (int i = 0; i < documents.count(); i++)
    if (documents.at(i) == doc) {
      index = i;
      break;
    }

  if (index > -1) {
    if (index > 0)
      index--;
    else
      index = documents.count() - 1;
    doc = documents.at(index);
    if (doc->showDefault != 0)
      showMode = doc->showDefault;
    else {
      if (doc->tree != NULL)
	showMode = DRAW_TREE;
      else
	showMode = DRAW_STRUC;
    }
    replace->close();
    newViewWindow(doc, showMode, replace->x(), replace->y(), replace->width(), replace->height());
  }
}


void MainWindow::nextStructureSlot(ViewCanvas* replace) {
  Document* doc = replace->canvas->getDocument();
  int index = -1;
  int showMode;

  for (int i = 0; i < documents.count(); i++)
    if (documents.at(i) == doc) {
      index = i;
      break;
    }

  if (index > -1) {
    if (index < documents.count() - 1)
      index++;
    else
      index = 0;
    doc = documents.at(index);
    if (doc->showDefault != 0)
      showMode = doc->showDefault;
    else {
      if (doc->tree != NULL)
	showMode = DRAW_TREE;
      else
	showMode = DRAW_STRUC;
    }
    replace->close();
    newViewWindow(doc, showMode, replace->x(), replace->y(), replace->width(), replace->height());
  }
}


/** Called when the status of the socket connection has changed.
    @param status the new connection status (a <code>GDE_STATUS_xxxx</code> constant,
      see GDataExchange)
    @param hostname the name of the host Grisu is running on
    @param port the number of the port used in the connection. */
void MainWindow::connectionStateSlot(unsigned char status, char* hostname, int port) {
  char stateBuf[100];
  char hostBuf[500];

  switch (status) {
  case GDE_STATE_NO_CONNECTION:
    strcpy(stateBuf, "No connection");
    strcpy(hostBuf, "");
    break;
  case GDE_STATE_READY: 
    strcpy(stateBuf, "Ready for connection");
    sprintf(hostBuf, "%s, %d", hostname, port);
    break;
  case GDE_STATE_CONNECTED:
    strcpy(stateBuf, "Connected");
    sprintf(hostBuf, "%s, %d", hostname, port);
    break;
  }

  statusBar->changeItem(stateBuf, SID_CONNECTION_STATE);
  statusBar->changeItem(hostBuf, SID_CONNECTION_HOST);
}
 
