/*  $Id: options.cpp,v 1.35 2003/05/22 16:43:14 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of the <code>Options</code> class.

    @author Holger Wunsch */

#include <qdialog.h>
#include <kcolordialog.h>
#include <kfontdialog.h>
#include <kconfig.h>
#include "options.h"
#include "global.h"

#include "options.moc"

extern bool optionChanged;

Options::Options() {
  readOptionFromConfig(); 
}



Options::Options(Options& op) {
  copyFromLocal(op);
}


Options::~Options() {
}



void Options::copyFromLocal(Options &op) {
  normalFont= op.normalFont;
  textFont  = op.textFont;
  errorFont = op.errorFont;
  typeFont  = op.typeFont;
  treeFont  = op.treeFont;
  leafFont  = op.leafFont;
  arclabelFont = op.arclabelFont;
  
  normalColor     = op.normalColor;
  typeColor       = op.typeColor;
  backgroundColor = op.backgroundColor;
  textColor       = op.textColor;
  errorColor      = op.errorColor;
  treeColor       = op.treeColor;
  treeLineColor   = op.treeLineColor;
  leafColor       = op.leafColor;
  hiddenColor     = op.hiddenColor;
  markerColor     = op.markerColor;
  foundColor      = op.foundColor;
  differentNodeColor = op.differentNodeColor;
  arclabelColor   = op.arclabelColor;
  arclabelBgColor = op.arclabelBgColor;

  treeLineWidth = op.treeLineWidth;  

  offset=op.offset;
  
  drawHidden=op.drawHidden;
  
  fitToDrawing=op.fitToDrawing;

  highlightStructures = op.highlightStructures;

  showAppName = op.showAppName;

  useKFileDialog = op.useKFileDialog;
}



void Options::changeColorSlot(int what, QWidget* parent) {
  QColor* col;
  switch(what) {
  case NormalColor:
    col = &normalColor;
    break;
  case TypeColor:
    col = &typeColor;
    break;
  case BackgroundColor:
    col = &backgroundColor;
    break;
  case TextColor:
    col = &textColor;
    break;
  case ErrorColor:
    col = &errorColor;
    break;
  case TreeColor:
    col = &treeColor;
    break;
  case TreeLineColor:
    col = &treeLineColor;
    break;
  case LeafColor:
    col = &leafColor;
    break;
  case HiddenColor:
    col = &hiddenColor;
    break;
  case MarkerColor:
    col = &markerColor;
    break;
  case FoundColor:
    col = &foundColor;
    break;
  case DifferentNodeColor:
    col = &differentNodeColor;
    break;
  case ArcLabelColor:
    col = &arclabelColor;
    break;
  case ArcLabelBgColor:
    col = &arclabelBgColor;
    break;
  default:
    col = NULL;
    break;
  }
  if (col) {
    KColorDialog kcol(parent);
    kcol.setColor(*col);
    if (kcol.getColor(*col) == QDialog::Accepted)
      optionChanged = true;
  }
}

void Options::changeFontSlot(int what) {
  QFont *fon;
  switch(what) {
  case NormalFont:
    fon = &normalFont;
    break;
  case TypeFont:
    fon = &typeFont;
    break;
  case TextFont:
    fon = &textFont;
    break;
  case ErrorFont:
    fon = &errorFont;
    break;
  case TreeFont:
    fon = &treeFont;
    break;
  case LeafFont:
    fon = &leafFont;
    break;
  case ArcLabelFont:
    fon = &arclabelFont;
    break;
  default:
    fon= NULL;
    break;
  }
  if (fon) {
    KFontDialog kfon;
    kfon.setFont(*fon);
    if (kfon.getFont(*fon) == QDialog::Accepted)
      optionChanged = true;
  }
}


void Options::readOptionFromConfig() {
  KConfig *conf= KApplication::kApplication()->config();
  
  conf->sync();
  conf->setGroup("Colors");
  
  QColor* defaultColor = new QColor();

  defaultColor->setRgb(255, 255, 255);
  backgroundColor = conf->readColorEntry("backgroundColor" , defaultColor);

  defaultColor->setRgb(0, 0, 0);
  normalColor     = conf->readColorEntry("normalColor"     , defaultColor);

  defaultColor->setRgb(0, 255, 0);
  textColor       = conf->readColorEntry("textColor"       , defaultColor);

  defaultColor->setRgb(0, 0, 255);
  typeColor       = conf->readColorEntry("typeColor"       , defaultColor);

  defaultColor->setRgb(255, 0, 0);
  errorColor      = conf->readColorEntry("errorColor"      , defaultColor);

  defaultColor->setRgb(0, 0, 0);
  treeColor       = conf->readColorEntry("treeColor"       , defaultColor);

  defaultColor->setRgb(0, 0, 0);
  treeLineColor   = conf->readColorEntry("treeLineColor"   , defaultColor);

  defaultColor->setRgb(128, 0, 0);
  leafColor       = conf->readColorEntry("leafColor"   , defaultColor);

  defaultColor->setRgb(192, 192, 192);
  hiddenColor     = conf->readColorEntry("hiddenColor", defaultColor);

  defaultColor->setRgb(255, 0, 0);
  markerColor     = conf->readColorEntry("markerColor", defaultColor);

  defaultColor->setRgb(0, 0, 255);
  foundColor      = conf->readColorEntry("foundColor", defaultColor);

  defaultColor->setRgb(255, 0, 0);
  differentNodeColor = conf->readColorEntry("differentNodeColor", defaultColor);

  defaultColor->setRgb(255, 0, 0);
  arclabelColor = conf->readColorEntry("arclabelColor", defaultColor);

  defaultColor->setRgb(230, 230, 230);
  arclabelBgColor = conf->readColorEntry("arclabelBgColor", defaultColor);

  delete defaultColor;

  conf->setGroup("Fonts");

  QFont* defaultFont = new QFont("helvetica", 8);
  
  normalFont      = conf->readFontEntry("normalFont" , defaultFont);
  textFont        = conf->readFontEntry("textFont"   , defaultFont);

  defaultFont->setItalic(true);
  typeFont        = conf->readFontEntry("typeFont"   , defaultFont);

  defaultFont->setItalic(false);
  defaultFont->setWeight(QFont::Bold);
  errorFont       = conf->readFontEntry("errorFont"  , defaultFont);
  treeFont        = conf->readFontEntry("treeFont"   , defaultFont);
  leafFont        = conf->readFontEntry("leafFont"   , defaultFont);

  defaultFont->setPointSize(8);
  defaultFont->setWeight(QFont::Normal);
  arclabelFont    = conf->readFontEntry("arclabelFont"   , defaultFont);

  delete defaultFont;

  conf->setGroup("Drawing");
  offset          = conf->readNumEntry("offset",2);
  drawHidden      = bool(conf->readNumEntry("drawHidden",int(false)));
  fitToDrawing    = bool(conf->readNumEntry("fitToDrawing",int(true)));
  treeLineWidth   = conf->readNumEntry("treeLineWidth",2);
  highlightStructures = bool(conf->readNumEntry("highlightStructures", int(false)));

  conf->setGroup("Global");
  writeConfig     = bool(conf->readNumEntry("writeConfig",int(true)));
  expandReentrancies = bool(conf->readNumEntry("expandReentrancies",int(true)));
  autoOpenWindows = bool(conf->readNumEntry("autoOpenWindows", int(true)));
  showAppName = bool(conf->readNumEntry("showAppName", int(false)));
  useKFileDialog = bool(conf->readNumEntry("useKFileDialog", int(false)));
  
}



void Options::writeOptionFromConfig() {
  KConfig *conf= KApplication::kApplication()->config();

  if (writeConfig) {
    conf->setGroup("Colors");
    conf->writeEntry("backgroundColor" , backgroundColor);
    conf->writeEntry("normalColor"     , normalColor);
    conf->writeEntry("textColor"       , textColor);
    conf->writeEntry("typeColor"       , typeColor);
    conf->writeEntry("errorColor"      , errorColor);
    conf->writeEntry("treeColor"       , treeColor);
    conf->writeEntry("leafColor"       , leafColor);
    conf->writeEntry("treeLineColor"   , treeLineColor);
    conf->writeEntry("hiddenColor"     , hiddenColor);
    conf->writeEntry("markerColor"     , markerColor);
    conf->writeEntry("foundColor"      , foundColor);
    conf->writeEntry("differentNodeColor", differentNodeColor);
    conf->writeEntry("arclabelColor"   , arclabelColor);
        
    
    conf->setGroup("Fonts");
    conf->writeEntry("normalFont"     , normalFont);
    conf->writeEntry("textFont"       , textFont);
    conf->writeEntry("typeFont"       , typeFont);
    conf->writeEntry("errorFont"      , errorFont);
    conf->writeEntry("treeFont"       , treeFont);
    conf->writeEntry("leafFont"       , leafFont);
    conf->writeEntry("arclabelFont"   , arclabelFont);
    
    
    conf->setGroup("Drawing");
    conf->writeEntry("offset"         , offset);
    conf->writeEntry("drawHidden"     , drawHidden);
    conf->writeEntry("fitToDrawing"   , fitToDrawing);
    conf->writeEntry("treeLineWidth"  , treeLineWidth);
    conf->writeEntry("highlightStructures" , highlightStructures);
    
    conf->setGroup("Global");
    conf->writeEntry("writeConfig", int(writeConfig));
    conf->writeEntry("expandReentrancies",int(expandReentrancies));
    conf->writeEntry("autoOpenWindows", int(autoOpenWindows));
    conf->writeEntry("showAppName", int(showAppName));
    conf->writeEntry("useKFileDialog", int(useKFileDialog));

    conf->sync();
  } else {
    conf->writeEntry("writeConfig", int(writeConfig));
    conf->sync();
  }
}


/** Writes all settings in the given printer dialog to the configuration file.
    @param printer the <code>QPrinter</code> object. */
void Options::writePrinterOptions(QPrinter& printer) {
  KConfig *conf= KApplication::kApplication()->config();

  conf->setGroup("Printer");
  conf->writeEntry("printerName", printer.printerName());
  conf->writeEntry("outputToFile", int(printer.outputToFile()));
  conf->writeEntry("outputFileName", printer.outputFileName());
  conf->writeEntry("orientation", int(printer.orientation()));
  conf->writeEntry("pageSize", int(printer.pageSize()));
  conf->writeEntry("numCopies", printer.numCopies());
  conf->writeEntry("colorMode", int(printer.colorMode()));

  conf->sync();
}


/** Initializes a printer dialog with options from the configuration file.
    @param printer the <code>QPrinter</code> object. */
void Options::readPrinterOptions(QPrinter& printer) {
  KConfig *conf= KApplication::kApplication()->config();

  if (conf->hasGroup("Printer")) {
    conf->setGroup("Printer");
    
    if (conf->hasKey("printerName"))
      printer.setPrinterName(conf->readEntry("printerName"));
    if (conf->hasKey("outputToFile"))
      printer.setOutputToFile(bool(conf->readNumEntry("outputToFile")));
    if (conf->hasKey("outputFileName"))
      printer.setOutputFileName(conf->readEntry("outputFileName"));
    if (conf->hasKey("orientation"))
      printer.setOrientation((QPrinter::Orientation) conf->readNumEntry("orientation"));
    if (conf->hasKey("pageSize"))
      printer.setPageSize((QPrinter::PageSize) conf->readNumEntry("pageSize"));
    if (conf->hasKey("numCopies"))
      printer.setNumCopies(conf->readNumEntry("numCopies"));
    if (conf->hasKey("colorMode"))
      printer.setColorMode((QPrinter::ColorMode) conf->readNumEntry("colorMode"));
  }
}
