/*  $Id: scrollcanvas.cpp,v 1.29 2003/05/22 16:43:14 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file
   
    Implementation of the <code>ScrollCanvas</code> class.

    @author Holger Wunsch */

#include "scrollcanvas.h"

#include "canvas.h"

ScrollCanvas::ScrollCanvas(Options* options, Document* d, GenericNode* rootNode, QWidget* parent) :
  QWidget(parent) {
  v_scroll = new MyScrollBar(QScrollBar::Vertical, this);
  h_scroll = new MyScrollBar(QScrollBar::Horizontal, this);

  canvas = new Canvas(options, v_scroll, h_scroll, d, rootNode, this);
}

ScrollCanvas::~ScrollCanvas() {
  delete v_scroll;
  delete h_scroll;
  delete canvas;
}

Canvas* ScrollCanvas::getCanvas() {
  return canvas;
}

QSize ScrollCanvas::size() const {
  QSize sz(canvas->width() + v_scroll->width(), canvas->height() + h_scroll->height());
  return sz;
}

void ScrollCanvas::resizeEvent(QResizeEvent *ev) {
  QWidget::resizeEvent(ev);
  
  canvas->setGeometry( 0, 0, width() - v_scroll->width() - 2, height() - h_scroll->height() - 2);
  
  v_scroll->setGeometry( canvas->x() + canvas->width()+2,
			 canvas->y(),
			 16,
			 canvas->height()
			 );
  h_scroll->setGeometry( canvas->x(),
			 canvas->y() + canvas->height()+2,
			 canvas->width(),
			 16
			 );
  
  QSize si=canvas->getMaxSize();
  int rx= si.width()  - canvas->width();
  int ry= si.height() - canvas->height();
  if ( rx < 1) rx = 1;
  if ( ry < 1) ry = 1;
  
  h_scroll->setRange((int)0 , rx);
  v_scroll->setRange((int)0 , ry);
  h_scroll->setSteps(10, canvas->width());
  v_scroll->setSteps(10, canvas->height());
  canvas->resetScrollbar();
}
