/*  $Id: slider.h,v 1.33 2003/05/22 16:43:14 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file
    Declaration of the MyScrollBar class.
    @author Carsten He */

#ifndef __SLIDER_H__
#define __SLIDER_H__

#include <qscrbar.h>
#include <qwidget.h> 

#include "data.h"
#include "options.h"


/** The MyScrollBar allows public access to the mouse event functions, which are private in
    QScrollBar. See also the documentation of <code>QScrollBar</code> for more information.
    @author Carsten He */
class MyScrollBar:public QScrollBar {
  Q_OBJECT

public:
   
  /** Creates a new scrollbar.
      @param parent the parent widget
      @param name name of this scrollbar (internal use only) */
  MyScrollBar( QWidget *parent=0, const char *name=0 );


  /** Creates a new scrollbar with a given orientation.
      @param o the scrollbar's orientation (<code>Qt::Horizontal</code> or
        <code>Qt::Vertical</code>).
      @param parent the parent widget
      @param name name of this scrollbar (internal use only) */
  MyScrollBar( Orientation o, QWidget *parent=0, const char *name=0 );


  /** Creates a new scrollbar, allowing the caller to specify most parameters
      of the scrollbar.
      @param minValue the scrollbar's minimum value.
      @param maxValue the scrollbar's maximum value.
      @param LineStep the scrollbar's line step size.
      @param PageStep the scrollbar's page step size.
      @param value the initial scrollbar value (initial thumb position).
      @param o the scrollbar's orientation (<code>Qt::Horizontal</code> or
        <code>Qt::Vertical</code>).
      @param parent the parent widget
      @param name name of this scrollbar (internal use only) */
  MyScrollBar( int minValue, int maxValue, int LineStep, int PageStep,
 	       int value, Orientation o,
 	       QWidget *parent=0, const char *name=0 );


  /** Called when the user moves the mouse over the scrollbar area.
      @param mo the mouse event. */
  void mouseMoveEvent(QMouseEvent *mo);


  /** Called when the user presses a mouse button in the scrollbar area.
      @param mo the mouse event. */
  void mousePressEvent(QMouseEvent *mo);


  /** Called when the user releases a mouse button in the scrollbar area.
      @param mo the mouse event. */
  void mouseReleaseEvent(QMouseEvent *mo);
};


#endif
