/*  $Id: yylex.cpp,v 1.33 2003/05/22 16:43:15 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "node.h"
#include "il.hpp"
#include "dataparser.h"

#define PARSE_ALL  0
#define PARSE_ATOM 1
#define PARSE_STRING 2
#define PARSE_INTEGER 3

unsigned char mode;

#define KAVM_DEBUG

#define RETBUF_SIZE 255

/** The location of the next token to be returned. Note that this variable must be
    set to 0 in the calling function (that is, DataParser.parse() in dataparser.cpp */
int curLoc;

/** This function increaes the buffer pointer and the curLoc variable by delta.
    @param delta the amount by which the pointer is to be increased.
    @param parg a pointer to the ParserArgs structure that contains the ioBuffer pointer. */
void incBufPtr(int delta, ParserArgs* parg) {
  parg->ioBuffer += delta;
  curLoc += delta;
}


int yylex(YYSTYPE* lvalp, void* arg)
{
  char retBuf[RETBUF_SIZE];
  unsigned short retIndex;
  unsigned char escape;

  mode = PARSE_ALL;

  while (((ParserArgs*) arg)->ioBuffer != 0x00) {
    if (mode == PARSE_ALL) {
      if (strncmp(((ParserArgs*) arg)->ioBuffer, "!newdata(tree)", 14) == 0) {
	incBufPtr(14, (ParserArgs*) arg);
	return NEWDATA_TREE;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "!newdata(structure)", 19) == 0) {
	incBufPtr(19, (ParserArgs*) arg);
	return NEWDATA_STRUCTURE;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "!newdata", 8) == 0) {
	incBufPtr(8, (ParserArgs*) arg);
	return NEWDATA;
      }
      else if (islower(*((ParserArgs*) arg)->ioBuffer)) {
        mode = PARSE_ATOM;
        retIndex = 0;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == '\"') {
	mode = PARSE_STRING;
	retIndex = 0;
	escape = 0;
	incBufPtr(1, (ParserArgs*) arg);
      }
      else if (isdigit(*((ParserArgs*) arg)->ioBuffer)) {
	mode = PARSE_INTEGER;
	retIndex = 0;
      }
      else if ((*((ParserArgs*) arg)->ioBuffer == ' ') || (*((ParserArgs*) arg)->ioBuffer == '\t')) {
	incBufPtr(1, (ParserArgs*) arg);
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'S') {
	incBufPtr(1, (ParserArgs*) arg);
	return STRUC_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'T') {
	incBufPtr(1, (ParserArgs*) arg);
	return TREE_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'R') {
	incBufPtr(1, (ParserArgs*) arg);
	return REENT_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'V') {
	incBufPtr(1, (ParserArgs*) arg);
	return FEATVAL_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'O') {
	incBufPtr(1, (ParserArgs*) arg);
	return DISJUNCTION_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'C') {
	incBufPtr(1, (ParserArgs*) arg);
	return CONJUNCTION_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'L') {
	incBufPtr(1, (ParserArgs*) arg);
	return LIST_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'M') {
	incBufPtr(1, (ParserArgs*) arg);
	return SET_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'F') {
	incBufPtr(1, (ParserArgs*) arg);
	return FUNCTION_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'D') {
	incBufPtr(1, (ParserArgs*) arg);
	return REL_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'A') {
	incBufPtr(1, (ParserArgs*) arg);
	return ATOM_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'Z') {
	incBufPtr(1, (ParserArgs*) arg);
	return TAIL_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == 'Y') {
	incBufPtr(1, (ParserArgs*) arg);
	return REST_MARKER;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == '\n') {
	incBufPtr(1, (ParserArgs*) arg);
	return 0;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == '(') {
	incBufPtr(1, (ParserArgs*) arg);
	return LEFT_PAREN;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == ')') {
	incBufPtr(1, (ParserArgs*) arg);
	return RIGHT_PAREN;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "+", 1) == 0) {
	incBufPtr(1, (ParserArgs*) arg);
	return HIDDEN_FLAG;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "*", 1) == 0) {
	incBufPtr(1, (ParserArgs*) arg);
	return DIFFERENT_FLAG;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "-", 1) == 0) {
	incBufPtr(1, (ParserArgs*) arg);
	return STRIKEOUT_FLAG;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, "|", 1) == 0) {
	incBufPtr(1, (ParserArgs*) arg);
	return MULTILINE_FLAG;
      }
      else if (strncmp(((ParserArgs*) arg)->ioBuffer, ">", 1) == 0) {
	incBufPtr(1, (ParserArgs*) arg);
	return EXPANDED_FLAG;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == '/') {
	incBufPtr(1, (ParserArgs*) arg);
	return SLASH;
      }
      else if (*((ParserArgs*) arg)->ioBuffer == '#') {
	incBufPtr(1, (ParserArgs*) arg);
	return HASH;
      }
      else
	return ERROR;
    }
    else if (mode == PARSE_ATOM) {
      if (isalnum(*((ParserArgs*) arg)->ioBuffer) || (*((ParserArgs*) arg)->ioBuffer == '_')) {
	retBuf[retIndex] = *((ParserArgs*) arg)->ioBuffer;
	retIndex++;
	((ParserArgs*) arg)->ioBuffer++;
#ifdef DEBUG_KAVM
	if (retIndex > RETBUF_SIZE) {
	  printf("Return buffer overflow in yylex\n");
	  return 0;
	}
#endif
      }
      else {
	retBuf[retIndex] = 0x00;
	lvalp->string_type = (char*) malloc(strlen(retBuf) + 1);
	strcpy(lvalp->string_type, retBuf);
	return ATOM;
      }
    }
    else if (mode == PARSE_STRING) {
      if ((*((ParserArgs*) arg)->ioBuffer == '\"') && (escape == 0)) {
	retBuf[retIndex] = 0x00;
	lvalp->string_type = (char*) malloc(strlen(retBuf) + 1);
	strcpy(lvalp->string_type, retBuf);
	incBufPtr(1, (ParserArgs*) arg);
	return STRING;
      }
      else {
	// If the buffer is too small to hold the complete string,
	// leave room for the string terminator and ignore all
	// subsequent characters
	if (retIndex >= RETBUF_SIZE - 2) {
	  incBufPtr(1, (ParserArgs*) arg);
	  continue;
	}
	
	if (*((ParserArgs*) arg)->ioBuffer == '\\') {
	  if (escape == 0) {
	    escape = 1;
	    incBufPtr(1, (ParserArgs*) arg);
	    continue;
	  }
	}
	escape = 0;
	retBuf[retIndex] = *((ParserArgs*) arg)->ioBuffer;
	retIndex++;
	incBufPtr(1, (ParserArgs*) arg);

#ifdef DEBUG_KAVM
	if (retIndex > RETBUF_SIZE) {
	  printf("Return buffer overflow in yylex\n");
	  return 0;
	}
#endif
      }
    }
    else if (mode == PARSE_INTEGER) {
      if (!isdigit(*((ParserArgs*) arg)->ioBuffer)) {
	retBuf[retIndex] = 0x00;
	lvalp->int_type = strtol(retBuf, NULL, 10);
	return INTEGER;
      }
      retBuf[retIndex] = *((ParserArgs*) arg)->ioBuffer;
      retIndex++;
      incBufPtr(1, (ParserArgs*) arg);
    }
  }
}
